package lab;

import lab.storage.DbConnector;
import lab.storage.ScoreStorageInterface;

public class Config {

	private static Config instance;

	private ScoreStorageInterface scoreStorageInterface;
	private double monsterMinXPopsition;
	private double monsterMinSpeed;
	private double monsterMaxSpeed;
	private double obstacleWidth;
	private double obstacleHeight;
	private double playerStartSpeed;
	private int monsterMultiplier;

	public static void configure(Config setting) {
		instance = setting;
	}

	private Config(ScoreStorageInterface scoreStorageInterface, double monsterMinXPopsition, double monsterMinSpeed,
			double monsterMaxSpeed, double obstacleWidth, double obstacleHeight, double playerStartSpeed,
			int monsterMultiplier) {
		super();
		this.scoreStorageInterface = scoreStorageInterface;
		this.monsterMinXPopsition = monsterMinXPopsition;
		this.monsterMinSpeed = monsterMinSpeed;
		this.monsterMaxSpeed = monsterMaxSpeed;
		this.obstacleWidth = obstacleWidth;
		this.obstacleHeight = obstacleHeight;
		this.playerStartSpeed = playerStartSpeed;
		this.monsterMultiplier = monsterMultiplier;
	}

	public static Config getInstance() {
		return instance;
	}

	public ScoreStorageInterface getScoreStorageInterface() {
		return scoreStorageInterface;
	}

	public double getMonsterMinXPopsition() {
		return monsterMinXPopsition;
	}

	public double getMonsterMinSpeed() {
		return monsterMinSpeed;
	}

	public double getMonsterMaxSpeed() {
		return monsterMaxSpeed;
	}

	public double getObstacleHeight() {
		return obstacleHeight;
	}

	public double getObstacleWidth() {
		return obstacleWidth;
	}

	public double getPlayerStartSpeed() {
		return playerStartSpeed;
	}

	public int getMonsterMultiplier() {
		return monsterMultiplier;
	}

	public static Builder builder() {
		return new Builder();
	}

	public static Config getInstanceForHardcoreGame() {
		return builder().monsterMinSpeed(150).monsterMaxSpeed(500).monsterMultiplier(5).monsterMinXPopsition(0.2)
				.build();
	}

	public static class Builder {
		private ScoreStorageInterface scoreStorageInterface = new DbConnector();
		private double monsterMinXPopsition = 0.5;
		private double monsterMinSpeed = 50;
		private double monsterMaxSpeed = 150;
		private double obstacleWidth = 30;
		private double obstacleHeight = 20;
		private double playerStartSpeed = 50;
		private int monsterMultiplier = 1;

		public Builder scoreStorageInterface(ScoreStorageInterface scoreStorageInterface) {
			this.scoreStorageInterface = scoreStorageInterface;
			return this;
		}

		public Builder monsterMinXPopsition(double monsterMinXPopsition) {
			this.monsterMinXPopsition = monsterMinXPopsition;
			return this;
		}

		public Builder monsterMinSpeed(double monsterMinSpeed) {
			this.monsterMinSpeed = monsterMinSpeed;
			return this;
		}

		public Builder monsterMaxSpeed(double monsterMaxSpeed) {
			this.monsterMaxSpeed = monsterMaxSpeed;
			return this;
		}

		public Builder obstacleWidth(double obstacleWidth) {
			this.obstacleWidth = obstacleWidth;
			return this;
		}

		public Builder obstacleHeight(double obstacleHeight) {
			this.obstacleHeight = obstacleHeight;
			return this;
		}

		public Builder playerStartSpeed(double playerStartSpeed) {
			this.playerStartSpeed = playerStartSpeed;
			return this;
		}

		public Builder monsterMultiplier(int monsterMultiplier) {
			this.monsterMultiplier = monsterMultiplier;
			return this;
		}

		public Config build() {
			return new Config(scoreStorageInterface, monsterMinXPopsition, monsterMinSpeed, monsterMaxSpeed,
					obstacleWidth, obstacleHeight, playerStartSpeed, monsterMultiplier);
		}

	}

}
